// src/useProgressiveCallback.ts
import { useCallback as useCallback2 } from "react";

// src/useMountSafeState.ts
import { useState, useRef, useEffect, useCallback } from "react";
function useMountSafeState(initialState) {
  let [state, setState] = useState(initialState);
  let mounted = useRef(false);
  useEffect(() => {
    mounted.current = true;
    return () => {
      mounted.current = false;
    };
  }, []);
  let setMountSafeState = useCallback((state2) => {
    if (mounted.current)
      setState(state2);
  }, []);
  return [state, setMountSafeState];
}

// src/useProgressiveCallback.ts
function useProgressiveCallback(callback, deps) {
  let [pending, setPending] = useMountSafeState(void 0);
  let [error, setError] = useMountSafeState(void 0);
  let enhancedCallback = useCallback2(
    // @ts-ignore `args` are of the same type as in `callback`
    (...args) => {
      try {
        setError(void 0);
        let value = callback(...args);
        if (value instanceof Promise) {
          setPending(true);
          return value.catch((error2) => {
            setError(error2);
          }).finally(() => {
            setPending(false);
          });
        } else {
          setPending(false);
          return value;
        }
      } catch (error2) {
        setPending(false);
        setError(error2);
      }
    },
    deps
  );
  return [pending, enhancedCallback, error];
}
export {
  useProgressiveCallback
};
